import router from './router'
import store from './store'
import { Message } from 'element-ui'
import NProgress from 'nprogress' // progress bar
import 'nprogress/nprogress.css' // progress bar style
import { getToken } from '@/utils/auth'
import getPageTitle from '@/utils/get-page-title'

NProgress.configure({ showSpinner: false }) // NProgress Configuration

const whiteList = ['/login', '/auth-redirect', '/sso', '/formShortLink'] // no redirect whitelist

router.beforeEach(async (to, from, next) => {
  // start progress bar
  NProgress.start()

  // set page title
  document.title = getPageTitle(to.meta.title, to.meta.zhTitle)
  const hasToken = getToken()
  console.log(hasToken)
  console.log(to)


  if (to.query.code && to.path !== '/login' && hasToken) {
    await store.dispatch('user/resetToken')
    next(`/login?code=${to.query.code}`);
  }


  if (to.path == '/workFlowDetail' && to.query.token && hasToken != to.query.token) {
    let param = {
      token: to.query.token
    }
    store.dispatch('user/setToken', param).then(() => {
      next({
        ...to,
        replace: true
      })
    })
  }
  // determine whether the user has logged in
  if (hasToken && to.path !== '/sso' && to.path !== '/formShortLink') {

    if (store.getters.isLock && to.path !== '/lockScreen' && to.path !== '/login') {
      next({
        path: '/lockScreen'
      })
      NProgress.done()
    } else if (to.path === '/login') {
      // if is logged in, redirect to the home page
      next({
        path: '/'
      })
      NProgress.done()
    } else {
      const hasMenu = store.getters.menuList && store.getters.menuList.length > 0
      if (hasMenu) {
        next()
      } else {
        try {
          // get user info
          let res = await store.dispatch('user/getInfo')
          const accessRoutes = await store.dispatch('permission/generateRoutes', res)
          // dynamically add accessible routes
          router.addRoutes(accessRoutes)
          // hack method to ensure that addRoutes is complete
          // set the replace: true, so the navigation will not leave a history record
          next({
            ...to,
            replace: true
          })
        } catch (error) {
          // remove token and go to login page to re-login
          await store.dispatch('user/resetToken')
          if (error && typeof (error) == 'string') Message.error(error || 'Has Error')
          next(`/login?redirect=${to.path}`)
          NProgress.done()
        }
      }
    }
  } else {
    /* has no token*/
    if (whiteList.indexOf(to.path) !== -1) {
      // in the free login whitelist, go directly
      next();
    } else {
      // other pages that do not have permissionEle to access are redirected to the login page.
      if (to.query.code) {
        next(`/login?code=${to.query.code}`);
      } else {
        next(`/login?redirect=${to.path}`);
      }

      NProgress.done();
    }
  }
});

router.afterEach(() => {
  // finish progress bar
  NProgress.done()
})